// server.js
// where your node app starts

// init project
const express = require('express');
const doT = require('dot');
const moment = require('moment');


let posttemp = doT.template("<li>{{=it.name}}</li>");
let feedtemp ='';
let indextemp ='';
let postpagetemp ='';
const fs = require('fs');

const titles={"name":"my quick blog","summary":"because indieweb"};
 
fs.readFile('views/post.html', 'utf8', function(err, contents) {
    posttemp = doT.template(contents);
});
fs.readFile('views/feed.html', 'utf8', function(err, contents) {
    feedtemp = doT.template(contents);
});
fs.readFile('views/newindex.html', 'utf8', function(err, contents) {
    indextemp = doT.template(contents);
});
fs.readFile('views/postpage.html', 'utf8', function(err, contents) {
    postpagetemp = doT.template(contents);
});
// setup a new database
// persisted using async file storage
// Security note: the database is saved to the file `db.json` on the local filesystem.
// It's deliberately placed in the `.data` directory which doesn't get copied if someone remixes the project.
const low = require('lowdb')
const FileSync = require('lowdb/adapters/FileSync')
const adapter = new FileSync('.data/db.json')
const db = low(adapter)

const cookieSession = require('cookie-session')

const consumerKey = process.env.TWITTER_CONSUMER_KEY;
const consumerSecret = process.env.TWITTER_CONSUMER_SECRET;
const baseurl = 'https://'+process.env.PROJECT_DOMAIN+'.glitch.me/';

const appRoot = baseurl;
console.log("app root is " +appRoot);
console.log("callback is " +appRoot+ "/auth/twitter/callback");
const OAuth = require('oauth').OAuth
  , oauth = new OAuth(
      "https://api.twitter.com/oauth/request_token",
      "https://api.twitter.com/oauth/access_token",
      consumerKey,
      consumerSecret,
      "1.0",
      appRoot + "auth/twitter/callback",
      "HMAC-SHA1"
    );
const twitter = require('twitter-api').createClient();
const app = express();

app.use(cookieSession({
  name: 'session',
  keys: [process.env.COOKIE_KEY],

}))

app.get('/auth/twitter', function(req, res) {
 
  oauth.getOAuthRequestToken(function(error, oauth_token, oauth_token_secret, results) {
    if (error) {
      console.log(error);
      res.send("Authentication Failed!");
    }
    else {
      req.session.oauth = {
        token: oauth_token,
        token_secret: oauth_token_secret
      };
      //console.log(req.session.oauth);
      res.redirect('https://twitter.com/oauth/authenticate?oauth_token='+oauth_token)
    }
  });
 
});

app.get('/auth/twitterlogout', function(req, res) {
  // Only destroys the session, doesn't revoke the key.
  // Twitter doesn't have an API for user-level revocation.
  req.session = null;
  res.redirect('/');
});

app.get('/auth/twitter/callback', function(req, res, next) {
 
  if (req.session.oauth) {
    req.session.oauth.verifier = req.query.oauth_verifier;
    const oauth_data = req.session.oauth;
 
    oauth.getOAuthAccessToken(
      oauth_data.token,
      oauth_data.token_secret,
      oauth_data.verifier,
      function(error, oauth_access_token, oauth_access_token_secret, results) {
        if (error) {
          console.log(error);
          res.send("Authentication Failure!");
        }
        else {
          req.session.oauth.access_token = oauth_access_token;
          req.session.oauth.access_token_secret = oauth_access_token_secret;
          //console.log(results, req.session.oauth);
          //res.send("Authenticated <a href='/showuser'>as</a>");
          twitter.setAuth ( 
                consumerKey,
                consumerSecret, 
                req.session.oauth.access_token,
                req.session.oauth.access_token_secret 
            );

            twitter.get( 'account/verify_credentials', { skip_status: true }, function( user, error, status ){
                console.log( user ? 'Authenticated as @'+user.screen_name : 'Not authenticated' );
               req.session.user = user;
               //console.log(req.session.user);
               //res.send("Logged in as @"+user.screen_name);
               res.redirect('/'); 
            } );
        }
      }
    );
  }
  else {
    res.redirect('/'); // Redirect to login page
  }
 
});

app.get('/sendtweet', function(req, res, next) {
    //console.log("sendtweet: " + req.query.status +" lastid:"+ req.query.lastid );
	twitter.setAuth ( 
        consumerKey,
        consumerSecret, 
        req.session.oauth.access_token,
        req.session.oauth.access_token_secret 
    );
	if ( twitter.hasAuth() ) {
	   twitter.post('statuses/update',{'status':req.query.status,'in_reply_to_status_id':req.query.lastid}, function( tweet, error, status ){
					console.log( tweet ? 'posted as @'+tweet.user.screen_name : status+" "+error.message );               
				   res.send(tweet ? "<a href='https://twitter.com/" +tweet.user.screen_name+"/status/"+
						tweet.id_str+"'>"+tweet.text+"</a>":status+ + error.message );
				} );
    } else {
        res.send("<a href='/auth/twitter'>login first </a>");
    } 
        
});

app.get('/showuser', function(req, res, next) {
    if (req.session.user) {
	const data = {"profileImage":req.session.user.profile_image_url_https, "screenName": req.session.user.screen_name, "fullName": req.session.user.name };
	res.send(data);
    } else {
         res.send("not logged in");
    }
});


const micropub = require('micropub-express');
 
// Attach the micropub endpoint to "/micropub" or wherever else you want
app.use('/micropub', micropub({
 
  // Specify what endpoint you want to verify a token with and what the expected identity returned is
  tokenReference: {
    me: baseurl,
    endpoint: 'https://tokens.indieauth.com/token',
  },
 
  // And lastly: Do something with the created micropub document
  handler: function (micropubDocument, req) {
    // Do something with the micropubDocument and return a Promise to communicate status of the handling
    console.log(micropubDocument);
  const d = new Date();
  const isodate = d.toISOString();
  const url = isodate.replace('T','').replace(/:/g,'').split('.')[0]+'.html';
  let post = micropubDocument.properties;
  post['published']=isodate.replace('T',' ');
  post['url']=url;
  db.get('posts') 
    .push(post)
    .write()
    return Promise.resolve().then(function () {
	twitter.setAuth ( 
        consumerKey,
        consumerSecret, 
        req.session.oauth.access_token,
        req.session.oauth.access_token_secret 
    );
	if ( twitter.hasAuth() ) {
	   twitter.post('statuses/update',{'status':micropubDocument.properties.content[0],'in_reply_to_status_id':req.query.lastid}, function( tweet, error, status ){
					console.log( tweet ? 'posted as @'+tweet.user.screen_name : status+" "+error.message );               
			  } );
      }
      return { 'url': 'note/'+url };
    });
  }
}));

const data_defaults= { posts: [
      {"url":"1.html", "name":"This is an example post","content":"<b>with style</b> if you like","published":"2018-08-17 11:50+0100"},
    ]
  };
// default post list
db.defaults(data_defaults).write();

// http://expressjs.com/en/starter/static-files.html
app.use(express.static('public'));

// http://expressjs.com/en/starter/basic-routing.html
app.get("/", function (request, response) {
  response.sendFile(__dirname + '/views/index.html');
  let page=titles;
  let feed=titles;
  let list="";
  const posts = db.get('posts').sortBy('published').reverse().value() // Find all posts in the collection
  posts.forEach(function(post) {
    post.datetext = moment(post.published).fromNow();
    list=list+posttemp(post); 
  });
  feed['posts'] =list;
  page['feed'] = feedtemp(feed);
  response.send(indextemp(page));
});

app.get("/note/:path", function (request,response) {
  const path = request.params['path'];
  const post = db.get('posts').find({url:path}).value() // Find this post
  let page=titles;
  post.datetext = moment(post.published).fromNow();
  page['post'] = posttemp(post);
  response.send(postpagetemp(page));  
});

app.get("/posts", function (request, response) {
  let dbposts=[];
  const posts = db.get('posts').reverse().value() // Find all posts in the collection
  posts.forEach(function(post) {
    dbposts.push([post.url,post.name,post.published]); // adds their info to the dbposts value
  });
  response.send(posts); // sends dbposts back to the page
});

// creates a new entry in the posts collection with the submitted values
app.post("/post", function (request, response) {
  const d = new Date();
  const isodate = d.toISOString();
  const url = isodate.replace('T','').replace(/:/g,'').split('.')[0]+'.html';
  db.get('posts') 
    .push({ name: request.query.name, published: request.query.published || isodate.replace('T',' '), url:url })
    .write();
  console.log("New post inserted in the database:"+url);
	twitter.setAuth ( 
        consumerKey,
        consumerSecret, 
        request.session.oauth.access_token,
        request.session.oauth.access_token_secret 
    );
	if ( twitter.hasAuth() ) {
	   twitter.post('statuses/update',{'status':request.query.name,'in_reply_to_status_id':request.query.lastid}, function( tweet, error, status ){
					console.log( tweet ? 'posted as @'+tweet.user.screen_name : status+" "+error.message );               
			  } );
      }
  response.sendStatus(200);
});

// removes entries from posts and populates it with default posts
app.get("/reset", function (request, response) {
  // removes all entries from the collection
  db.get('posts')
  .remove()
  .write()
  console.log("Database cleared");
  data_defaults['posts'].forEach(function(post){
    db.get('posts')
      .push(post)
      .write()
  });
  console.log("Default post added");
  
  response.redirect("/");
});

// removes all entries from the collection
// app.get("/clear", function (request, response) {
//   // removes all entries from the collection
//   db.get('posts')
//   .remove()
//   .write()
//   console.log("Database cleared");
//   response.redirect("/");
// });

// listen for requests :)
const listener = app.listen(process.env.PORT, function () {
  console.log('Your app is listening on port ' + listener.address().port);
});